/*
Presto - An Allegro tilemap library
Copyright (C) 2008  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

presto.h: engine variables and functions
*/


#include <stdio.h>
#include <allegro.h>
#include "loadpng.h"


//NOTE: ALLEGRO MUST BE INITIALIZED BEFORE USING THESE LIBRARY FUNCTIONS

//Please pay attention to which struct elements are labeled as PRIVATE in the comments.
//These are used internally and should not be changed by a user.


typedef struct {
	unsigned int index; //which frame of animation the tile is on for animations
	int tileid; //the id of the bitmap from the pool this tile is currently displaying
	int orderid; //id of the order/delay to follow.  -1 for no animation
	volatile int loop; //whether to loop the animation or not (used when the tile is changed to an explosion or something.  -1==loop infinitely, >0 == loop X times.  This variable is deincremented after every loop if not -1.
	unsigned int wait;  //PRIVATE: how many timer triggers this tile has been waiting to switch frames
	unsigned int sync_to_frameclock; //1 to sync to the animation frameclock (and thereby be in sync with other tiles with the same animation), 0 to not
} TILE;



typedef struct {
	unsigned int width;
	unsigned int height;
	unsigned int tile_w;
	unsigned int tile_h;

	unsigned int num_anims; //the number of animation sequences stored
	
	//these next siz elements make up a tile animation
	unsigned int **orders; //contains animation index information shared by all ANIM_TILES
	unsigned int **delays; //contains animation delay information shared by all ANIM_TILES
	int *loopvals; //PRIVATE: contains loop values corresponding to the orders and delays indeces.  Used for initializing tile loop values from a tilemap file and when changing animatinos
	unsigned int *num_frames; //the number of frames contained in each animation
	int *default_frame; //the bitmap to revert to after looping is done.  -1==stay on last frame of animation
	unsigned int *frameclock; //PRIVATE: global index for each animation that tiles can sync to
	unsigned int *wait; //PRIVATE: just like the TILE element only for the frameclock

	TILE *map; //a tilemap_w x tilemap_h array with a number corresponding to the index of the tile in the pool used
	BITMAP **pool; //array of BITMAP *'s for each tile to reference
	int *trans; //value from 0-255 that controls how each sprite in the pool is blended with the background.  0==transparent, 255==opaque, -1 to disregard transparency
	unsigned int poolsize; //how many tiles exist in the pool
	
} TILEMAP;


typedef struct {
	unsigned int num;
	TILEMAP **list;

} TILEMAP_LIST;


#define printi(x) fprintf(stderr,"VALUE:%d\n",x);

#define map(x,y) tilemap->map[(y*tilemap->width)+x]


unsigned long int frame_delay_gcd;

//contains the last error string produced by Presto
char tilemap_errstr[256+25+64];
char tilemap_fileerr[256+25];
int tilemap_err;




//Fills tilemap_err with a description of error code err after a Presto function errors
char *tilemap_get_err(void);

//Sets up the engine for use with animation tiles.  Does not have to be called if not using animation.
int install_tilemap(TILEMAP_LIST *list);

//frees up the timer for animation
void uninstall_tilemap(void);

//timer handler for animation
void anim_timer(void);

//loads a tilemap from a file where the first two values are the height and width of the map,
//followed by the tilemap data, then the sprite list.  Returns 0 on success, <0 on error
int load_tilemap(char *filename, TILEMAP *tilemap);

//frees data associated with the tilemap
void free_tilemap(TILEMAP *tilemap);


//renders to dest
void render_tilemap(BITMAP *dest, TILEMAP *tilemap, unsigned int x, unsigned int y, unsigned int width, unsigned int height);

//greatest common denomenator
unsigned int GCD(unsigned int a, unsigned int b);

//least common multiple (probably not worth the function overhead in size, so it's a macro
#define LCM(a,b) ((a*b)/GCD(a,b))

void update_anim_tiles(TILEMAP_LIST *list, unsigned int x, unsigned int y, unsigned int width, unsigned int height);

//Seeks to the next line of real data in a tilemap file
int next_line(FILE *file);


#include "functions.h"

